/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(Promise, log, service$schema, Random) {

    const _checkUniqueConstraints = Promise.method(function(coll, query) {
        if (query && query.$or.length > 0) {
            return coll.count(query).then(function(result) {
                return result === 0;
            });
        } else {
            return true;
        }
    });

    const _validateWithSchema = Promise.method(function(schema, db, collectionName, data) {
        if (schema) {
            return schema.validate(data).then((data) => {
                return schema.createUniqueConstraint(data).then(function(uniqueContext) {
                    let coll = db.collection(collectionName);

                    if (!data._id) {
                        data._id = Random.id();
                    }

                    return _checkUniqueConstraints(coll, uniqueContext.uniqueQuery).then((isUnique) => {
                        if (!isUnique) {
                            var err = new Error("not-unique");
                            err.isJoi = true;
                            err.details = {
                                uniqueKeys: uniqueContext.uniqueKeys
                            };
                            throw err;
                        } else {
                            return data;
                        }
                    });
                });
            });
        } else {
            return data;
        }
    });

    return Promise.method(function(db, collectionName, data, opts = {
        view: 'public'
    }) {
        var schema = service$schema(opts.schema || collectionName);

        return _validateWithSchema(schema, db, collectionName, data).then(function(data) {
            let coll = db.collection(collectionName);
            return coll.insertOne(data).then((result) => {
                data._id = result.insertedId;
                if (schema) {
                    return schema.view(data, opts.view);
                } else {
                    return data;
                }
            });

        });

    });

};