/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(Promise, _, config, log, service$schema) {

    return Promise.method(function(db, collectionName, filter, opts = {}) {
        log.debug("MongoList: %s", collectionName, filter);
        var coll = db.collection(collectionName);
        var cursor = coll.find(filter || {});

        opts.limit = opts.limit || 1000;
        opts.offset = opts.offset || 0;

        if (opts.limit) {
            cursor.limit(opts.limit);
        }

        if (opts.offset) {
            cursor.skip(opts.offset);
        }

        if (opts.sort) {
            let sort = {};

            if (_.isString(opts.sort)) {
                var criteria = opts.sort.split(',');
                _.each(criteria, function(c) {
                    if (c.indexOf('-') === 0) {
                        sort[c.substring(1)] = -1;
                    } else {
                        sort[c] = 1;
                    }
                });
            } else {
                sort = opts.sort;
            }
            cursor.sort(sort);
        }

        return Promise.join(
            coll.count(filter),
            cursor.toArray()
        ).spread((count, results) => {
            let schema = service$schema(opts.schema);

            // If we have a schema, apply it to all received elements
            return schema.validate(results).then(function(results) {

                return Promise.map(results, function(result) {
                    return schema.resolveRefs(db, result);
                }).then(function(results) {
                    // Create metadata to ease paging (if requested)
                    if (opts.meta) {
                        return {
                            data: results,
                            meta: {
                                total: count,
                                offset: opts.offset || 0,
                                limit: opts.limit,
                                page: Math.floor((opts.offset || 0) / opts.limit) + 1
                            }
                        };
                    } else {
                        return results;
                    }
                });

            });
        });
    });

};