/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(Promise, log, _, service$schema) {

    return Promise.method(function(db, collectionName, selector, opts = {}) {
        log.debug("MongoRead: %s", collectionName, selector);
        var coll = db.collection(collectionName);

        if (_.isString(selector)) {
            selector = {
                _id: selector
            };
        }

        return coll.findOne(selector || {}).then(function(result) {
            if (result) {
                let schema = service$schema(opts.schema);

                // If we have a schema, apply it to our entity
                return schema.validate(result).then(function(result) {
                    return schema.resolveRefs(db, result, opts).then(function(result) {
                        log.warn("OPTS", opts);
                        if (opts.view) {
                            return schema.view(result, opts.view);
                        } else {
                            return result;
                        }
                    });
                });
            }
        });
    });

};