/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
module.exports = function(Promise, _, service$schema) {
  return Promise.method(function(db, collectionName, query, opts = {
    type: 'fulltext',
    or: false,
    lang: 'en'
  }) {
    opts.type = opts.type || 'fulltext';
    opts.lang = opts.lang || 'en';

    if (opts.type === 'fulltext') {
      let coll = db.collection(collectionName);

      if (!opts.or) {
        query = _.map(query.split(' '), function(segment) {
          return '"' + segment + '"';
        }).join(' ');
      }

      return coll.find({
        $text: {
          $search: query,
          $language: opts.lang,
          $caseSensitive: opts.case || false,
          $diacriticSensitive: false
        }
      }, {
        score: {
          $meta: 'textScore'
        }
      }).sort({
        score: {
          $meta: 'textScore'
        }
      }).toArray();
    } else if (opts.type === 'spatial') {
      if (opts.schema) {
        return service$schema(opts.schema).getSpatialIndexSpec().then(function(spatial) {
          var coll = db.collection(collectionName);
          var q = {};

          q[_.keys(spatial)[0]] = {
            $nearSphere: {
              $geometry: {
                type: 'Point',
                coordinates: [query.longitude, query.latitude]
              },
              $maxDistance: query.radius * 1000
            }
          };

          return coll.find(q).toArray();
        });
      } else {
        return [];
      }
    } else if (opts.type === 'partial') {
      let coll = db.collection(collectionName);
      let cursor = coll.find(query);
      if (opts.limit) {
        cursor.limit(opts.limit);
      }

      if (opts.offset) {
        cursor.skip(opts.offset);
      }

      return cursor.toArray();
    } else {
      throw new Error('Unsupported search type:' + opts.type);
    }
  });
};