/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(Promise, log, service$schema, service$mongoRead, service$mongoList, _) {

    const _checkUniqueConstraints = Promise.method(function(coll, query) {
        if (query && query.$or.length > 0) {
            return coll.count(query).then(function(result) {
                return result === 0;
            });
        } else {
            return true;
        }
    });

    return Promise.method(function(db, collectionName, selector, data, opts = {}) {
        var schema = service$schema(opts.schema);

        return schema.validate(data).then((data) => {
            return schema.createUniqueConstraint(data, opts.skipConstraints).then(function(uniqueContext) {
                var coll = db.collection(collectionName);
                return _checkUniqueConstraints(coll, uniqueContext.uniqueQuery).then((isUnique) => {
                    if (isUnique) {
                        if (_.isString(selector)) {
                            selector = {
                                _id: selector
                            };
                        }

                        // Avoid updating _id
                        delete data._id;

                        return schema.updateRefs(db, data).then(function(result) {
                            return coll.update(selector, {
                                $set: result
                            }, {
                                safe: true,
                                multi: opts.multi,
                                upsert: opts.upsert
                            }).then((result) => {
                                if (result.result.n === 1) {
                                    return service$mongoRead(db, collectionName, selector, opts);
                                } else {
                                    return service$mongoList(db, collectionName, selector, opts);
                                }
                            });
                        });
                    } else {
                        var err = new Error("not-unique");
                        err.isJoi = true;
                        err.details = {
                            uniqueKeys: uniqueContext.uniqueKeys
                        };
                        throw err;
                    }
                });

            });

        });
    });

};